@@ -1,6 +1,7 @@
 """
 Helpers to traverse the AST, extract the docstring out, parse and format to intended style
 """
+
 from ast import (
     AnnAssign,
     Assign,
@@ -8,25 +9,38 @@ from ast import (
     ClassDef,
     FunctionDef,
     NodeTransformer,
+    arg,
     get_docstring,
     walk,
 )
 from collections import OrderedDict
 from copy import deepcopy
-from operator import attrgetter
+from operator import attrgetter, eq
 
-from cdd import emit, parse
+import cdd.emit.docstring
+import cdd.parse.docstring
+import cdd.parse.function
+from cdd.ast_cst_utils import (
+    find_cst_at_ast,
+    maybe_replace_doc_str_in_function_or_class,
+    maybe_replace_function_args,
+    maybe_replace_function_return_type,
+)
 from cdd.ast_utils import (
     annotate_ancestry,
     find_in_ast,
     maybe_type_comment,
     set_arg,
     set_docstring,
+    set_value,
     to_annotation,
     to_type_comment,
 )
+from cdd.cst_utils import reindent_block_with_pass_body
 from cdd.docstring_parsers import parse_docstring
-from cdd.parser_utils import ir_merge
+from cdd.parse.utils.parser_utils import ir_merge
+from cdd.pure_utils import PY_GTE_3_8, is_ir_empty, none_types, omit_whitespace
+from cdd.source_transformer import ast_parse
 
 
 def has_type_annotations(node):
@@ -125,9 +139,14 @@ class DocTrans(NodeTransformer):
 
         return Assign(
             targets=[node.target],
-            value=node.value,
-            lineno=None,
+            lineno=node.lineno,
+            col_offset=getattr(node, "col_offset", None),
+            end_lineno=getattr(node, "end_lineno", None),
+            end_col_offset=getattr(node, "end_col_offset", None),
             type_comment=to_type_comment(node.annotation),
+            # `var: int` is valid and turning it to `var = None  # type_comment int` would
+            # be wrong, as the upcoming smarter type tracer will reverse this to `var: Optional[int] = None`
+            value=set_value(none_types[-1]) if node.value is None else node.value,
         )
 
     def visit_Assign(self, node):
@@ -148,37 +167,22 @@ class DocTrans(NodeTransformer):
             assert len(node.targets) == 1
             return AnnAssign(
                 annotation=to_annotation(typ),
-                value=node.value,
-                lineno=None,
+                lineno=node.lineno,
+                col_offset=getattr(node, "col_offset", None),
+                end_lineno=getattr(node, "end_lineno", None),
+                end_col_offset=getattr(node, "end_col_offset", None),
                 simple=1,
                 target=node.targets[0],
                 expr=None,
                 expr_target=None,
                 expr_annotation=None,
-                **maybe_type_comment
+                **{} if node.value is None else {"value": node.value},
+                **maybe_type_comment,
             )
         else:
             setattr(node, "type_comment", typ)
         return node
 
-    def visit_Module(self, node):
-        """
-        visits the `Module`, potentially augmenting its docstring indentation
-
-        :param node: Module
-        :type node: ```Module```
-
-        :return: Potentially changed Module
-        :rtype: ```Module```
-        """
-        # Clean might be wrong if the header is a license or other long-spiel documentation
-        doc_str = get_docstring(node, clean=self.word_wrap is True)
-        empty = doc_str is None
-        if not empty:
-            set_docstring("\n{doc_str}\n".format(doc_str=doc_str), empty, node)
-        node.body = list(map(self.visit, node.body))
-        return node
-
     def visit_FunctionDef(self, node):
         """
         visits the `FunctionDef`, potentially augmenting its docstring and argument types
@@ -189,7 +193,7 @@ class DocTrans(NodeTransformer):
         :return: Potentially changed FunctionDef
         :rtype: ```FunctionDef```
         """
-        return self._handle_function(node, get_docstring(node, clean=True))
+        return self._handle_function(node, get_docstring(node, clean=False))
 
     def _get_ass_typ(self, node):
         """
@@ -209,78 +213,91 @@ class DocTrans(NodeTransformer):
                 {"typ": node.annotation or getattr(node, "type_comment", None)},
             )
         )
-        if not hasattr(node, "_location"):
+        if not hasattr(node, "_location") or node._location[:-1] == [None]:
             return typ_dict["typ"]
         search = node._location[:-1]
         search_str = ".".join(search)
 
-        self.memoized[search_str] = ir = (
-            self.memoized.get(
-                search_str,
-                (
-                    lambda parent: (
-                        (
-                            lambda doc_str: None
-                            if doc_str is None
-                            else parse.docstring(doc_str)
-                        )(get_docstring(parent, clean=True))
-                        if isinstance(parent, (ClassDef, AsyncFunctionDef, FunctionDef))
-                        else {"params": OrderedDict()}
-                    )
-                )(find_in_ast(search, self.whole_ast)),
-            )
-            or {"params": OrderedDict()}
-        )
+        self.memoized[search_str] = ir = self.memoized.get(
+            search_str,
+            (
+                lambda parent: (
+                    (
+                        lambda doc_str: None
+                        if doc_str is None
+                        else cdd.parse.docstring.docstring(doc_str)
+                    )(get_docstring(parent, clean=False))
+                    if isinstance(parent, (ClassDef, AsyncFunctionDef, FunctionDef))
+                    else {"params": OrderedDict()}
+                )
+            )(find_in_ast(search, self.whole_ast)),
+        ) or {"params": OrderedDict()}
 
         return ir["params"].get(name, typ_dict)[
             "typ"
         ]  # if ir is not None and ir.get("params") else None
 
-    def _handle_function(self, node, doc_str):
+    def _handle_function(self, node, original_doc_str):
         """
         Handle functions
 
         :param node: AsyncFunctionDef | FunctionDef
         :type node: ```Union[AsyncFunctionDef, FunctionDef]```
 
-        :param doc_str: The docstring
-        :type doc_str: ```Optional[str]```
+        :param original_doc_str: The docstring
+        :type original_doc_str: ```Optional[str]```
 
         :return: Same type as input with args, returns, and docstring potentially modified
         :rtype: ```Union[AsyncFunctionDef, FunctionDef]```
         """
-        ir = parse_docstring(doc_str)
-        ir_merge(ir, parse.function(node))
+        ir = parse_docstring(original_doc_str)
+        ir_merge(ir, cdd.parse.function.function(node))
         ir["name"] = node.name
         indent_level = max(
             len(node._location), 1
         )  # function docstrings always have at least 1 indent level
-        _doc_str = emit.docstring(
-            ir,
-            emit_types=not self.type_annotations,
-            emit_default_doc=False,
-            docstring_format=self.docstring_format,
-            indent_level=indent_level,
-            word_wrap=self.word_wrap,
+        doc_str = (
+            None
+            if is_ir_empty(ir)
+            else cdd.emit.docstring.docstring(
+                ir,
+                emit_types=not self.type_annotations,
+                emit_default_doc=False,
+                docstring_format=self.docstring_format,
+                indent_level=indent_level,
+                word_wrap=self.word_wrap,
+            )
         )
-        if not _doc_str or _doc_str.isspace():
-            if doc_str is not None:
+        if not doc_str or doc_str.isspace():
+            if original_doc_str is not None:
                 del node.body[0]
         else:
-            set_docstring(_doc_str, False, node)
+            set_docstring(
+                original_doc_str
+                if original_doc_str
+                and eq(*map(omit_whitespace, (original_doc_str, doc_str)))
+                else doc_str,
+                False,
+                node,
+            )
         if self.type_annotations:
             # Add annotations
             if ir["params"]:
                 node.args.args = list(
                     map(
-                        lambda _arg: set_arg(
-                            _arg.arg,
+                        lambda _arg: arg(
+                            arg=_arg.arg,
                             annotation=to_annotation(
                                 _arg.annotation
                                 if _arg.annotation is not None
                                 or _arg.arg in frozenset(("self", "cls"))
                                 else ir["params"][_arg.arg].get("typ")
                             ),
+                            identifier_arg=None,
+                            end_col_offset=getattr(_arg, "end_col_offset", None),
+                            **dict(expr=None, **maybe_type_comment)
+                            if PY_GTE_3_8
+                            else {},
                         ),
                         node.args.args,
                     )
@@ -316,4 +333,45 @@ def clear_annotation(node):
     return node
 
 
-__all__ = ["DocTrans", "clear_annotation", "has_type_annotations"]
+def doctransify_cst(cst_list, node):
+    """
+    Carefully replace only docstrings, function return annotations, assignment and annotation assignments.
+    (maintaining all other existing whitespace, comments, &etc.); and only when cdd has changed them
+
+    :param cst_list: List of `namedtuple`s with at least ("line_no_start", "line_no_end", "value") attributes
+    :type cst_list: ```List[NamedTuple]```
+
+    :param node: AST node with a `.body`, probably the `ast.Module`
+    :type node: ```AST```
+    """
+    for _node in walk(node):
+        if hasattr(_node, "_location"):
+            is_func = isinstance(_node, (AsyncFunctionDef, FunctionDef))
+            if isinstance(_node, ClassDef) or is_func:
+                cst_idx, cst_node = find_cst_at_ast(cst_list, _node)
+
+                if cst_node is not None:
+                    maybe_replace_doc_str_in_function_or_class(_node, cst_idx, cst_list)
+
+                    if is_func:
+                        cur_ast_node = ast_parse(
+                            reindent_block_with_pass_body(cst_list[cst_idx].value),
+                            skip_annotate=True,
+                            skip_docstring_remit=True,
+                        ).body[0]
+
+                        maybe_replace_function_return_type(
+                            _node, cur_ast_node, cst_idx, cst_list
+                        )
+                        maybe_replace_function_args(
+                            _node, cur_ast_node, cst_idx, cst_list
+                        )
+            # TODO: AnnAssign|Assign
+            # AnnAssign|Assign is separate task than the `maybe_replace_function_args` as inferring types is done
+            #   better with knowledge of function return types and function arguments (`default` being the only issue)
+            # elif isinstance(_node, (AnnAssign, Assign)):
+            #     print("(AnnAssign | Assign)._location:", _node._location, ";")
+            #     print_ast(_node)
+
+
+__all__ = ["DocTrans", "clear_annotation", "doctransify_cst", "has_type_annotations"]
