@@ -11,7 +11,8 @@ from itertools import chain, groupby
 from operator import itemgetter
 from os import makedirs, path
 
-from cdd import emit, parse
+import cdd.emit.file
+import cdd.parse.class_
 from cdd.ast_utils import maybe_type_comment, set_value
 from cdd.exmod_utils import emit_file_on_hierarchy, get_module_contents
 from cdd.pkg_utils import relative_filename
@@ -24,6 +25,8 @@ def exmod(
     blacklist,
     whitelist,
     output_directory,
+    mock_imports,
+    no_word_wrap,
     dry_run,
     filesystem_layout="as_input",
 ):
@@ -45,6 +48,12 @@ def exmod(
     :param output_directory: Where to place the generated exposed interfaces to the given `--module`.
     :type output_directory: ```str```
 
+    :param mock_imports: Whether to generate mock TensorFlow imports
+    :type mock_imports: ```bool```
+
+    :param no_word_wrap: Whether word-wrap is disabled (on emission).
+    :type no_word_wrap: ```Optional[Literal[True]]```
+
     :param dry_run: Show what would be created; don't actually write to the filesystem
     :type dry_run: ```bool```
 
@@ -59,9 +68,11 @@ def exmod(
                     module=module,
                     blacklist=blacklist,
                     whitelist=whitelist,
+                    mock_imports=mock_imports,
+                    filesystem_layout=filesystem_layout,
+                    no_word_wrap=no_word_wrap,
                     output_directory=output_directory,
                     dry_run=dry_run,
-                    filesystem_layout=filesystem_layout,
                 ),
                 emit_name or iter(()),
             ),
@@ -88,6 +99,9 @@ def exmod(
         import_module
     )(module)
 
+    # if module.__file__ is None:
+    #     raise ModuleNotFoundError(module_name)
+
     module_root_dir = path.dirname(module.__file__) + path.sep
 
     mod_path = ".".join((path.basename(module_root_dir[:-1]), module_name))
@@ -108,8 +122,10 @@ def exmod(
         emit_name=emit_name,
         module_name=module_name,
         new_module_name=new_module_name,
+        mock_imports=mock_imports,
         filesystem_layout=filesystem_layout,
         output_directory=output_directory,
+        no_word_wrap=no_word_wrap,
         dry_run=dry_run,
     )
 
@@ -127,7 +143,9 @@ def exmod(
                     )(relative_filename(getfile(name_source[1]))),
                     {"params": OrderedDict(), "returns": OrderedDict()}
                     if dry_run
-                    else parse.class_(name_source[1], merge_inner_function="__init__"),
+                    else cdd.parse.class_.class_(
+                        name_source[1], merge_inner_function="__init__"
+                    ),
                 ),
                 map(
                     lambda name_source: (
@@ -165,7 +183,7 @@ def exmod(
     if dry_run:
         print("write\t{init_filepath!r}".format(init_filepath=init_filepath))
     else:
-        emit.file(
+        cdd.emit.file.file(
             Module(
                 body=list(
                     chain.from_iterable(
